/**/
/* xamoeba_xpowell.c  -- C code is from NUMERICAL RECIPES IN C --
 *                       Additional code written by Keith Poole
 *                       September - October 2009
 *
   Amoeba -- Uses Nedler-Mead downhill simplex method to find a minimum/maximum
   of a multi-dimensional function
                 |->amotry->func 
Main -> amoeba --|
                 |->func

                 Amoeba initializes the simplex and calls func for each
                 point in the simplex.  The simplex starts at the usual
                 triangluar definition plus the origin -- ndim +1.

                 func is the user defined function being
                 minimized/maximized;

                 Amotry -- (from description in NUMERICAL RECIPES)
                 Extrapolates by a factor, fac, through the face of the
                 simplex across from a high point, tries, and replaces
                 the high point if the new point is better. */
/* Powell Routine:  
   Uses Powell's Quadratically Convergent Method to find a minimum/maximum
   of a multi-dimensional function
   
                 |->func 
Main -> powell --|          | ->mnbrak| 
                 |->linmin->|         |
                            |->brent->f1dim->func

                 Powell minimizes a function of n variables.  The
                 starting values are in the vector p[1:n] and an n by n
                 matrix of directions -- normally a simplex.

                 func is the user defined function being
                 minimized/maximized;

                 linmin -- Finds the minimum on a line joining p and xi.

		 mnbrak -- Used by linmin.  It finds three points that bracket the
                 minimum.

		 brent -- Given the 3 points from mnbrak, it finds the
				 minimum

				 */
/*
 * NUMERICAL DERIVATIVES

   Main -> dfridr -- This computes the first derivatives using simple
                    (f(x+h) - f(x-h))/2h
        -> dfridr2nd -- This computes the 2nd derivatives
                    df/dxdy = (f(x+h,y+h) - f(x+h,y-h) - f(x-h,y+h) + f(x-h,y-h))/(4*h*h)
		    d2f/dx2 = ( f(x+2h) + f(x-2h) - 2f(x) )/4h^2

***************************************************************************************
. summ

    Variable |       Obs        Mean    Std. Dev.       Min        Max
-------------+--------------------------------------------------------
     partyid |      1421    2.377903     2.06071          0          6
   incomecat |      1421    21.12386     7.76314         10         35
 incomequint |      1421    3.001407    1.415208          1          5
        race |      1421    .1491907    .3564018          0          1
         sex |      1421    .5622801    .4962807          0          1
-------------+--------------------------------------------------------
       south |      1421    .3019001    .4592438          0          1
   education |      1421    1.414497    .7154538          1          3
         age |      1421    46.00774    16.05494         20         90
       voted |      1421    1.231527    .9310102          0          3

. oprobit partyid incomequint race sex south education age

Iteration 0:   log likelihood = -2633.5601
Iteration 1:   log likelihood = -2494.5207
Iteration 2:   log likelihood = -2494.0641
Iteration 3:   log likelihood = -2494.0641

Ordered probit regression                         Number of obs   =       1421
                                                  LR chi2(6)      =     278.99
                                                  Prob > chi2     =     0.0000
Log likelihood = -2494.0641                       Pseudo R2       =     0.0530

------------------------------------------------------------------------------
     partyid |      Coef.   Std. Err.      z    P>|z|     [95% Conf. Interval]
-------------+----------------------------------------------------------------
 incomequint |   .0141518   .0226314     0.63   0.532    -.0302049    .0585086
        race |  -1.130843   .0903177   -12.52   0.000    -1.307862   -.9538234
         sex |   .0239784    .057164     0.42   0.675     -.088061    .1360178
       south |  -.3237482   .0632066    -5.12   0.000    -.4476307   -.1998656
   education |    .232688   .0417371     5.58   0.000     .1508848    .3144911
         age |   .0032912   .0018515     1.78   0.075    -.0003377      .00692
-------------+----------------------------------------------------------------
       /cut1 |  -.5671707   .1471578                     -.8555948   -.2787466
       /cut2 |   .2302907   .1451673                      -.054232    .5148135
       /cut3 |   .4976995   .1452729                      .2129699    .7824291
       /cut4 |    .794375    .146057                      .5081085    1.080642
       /cut5 |   1.084038     .14739                      .7951585    1.372917
       /cut6 |   1.735149   .1525477                      1.436161    2.034137
------------------------------------------------------------------------------

*/

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <R_ext/Lapack.h>
#include <R_ext/BLAS.h>
//
#define NDIM 12
#define NMAX 10000
#define ITMAX 2000
#define nrowX 8526
#define ncolX 12
#define FTOL 0.00000000001
#define TOL 0.000002
#define GOLD 1.618034
#define GLIMIT 100.0
#define TINY 1.0e-10
#define CON 1.4
#define BIG 1.0e30
#define NTAB 10
#define SAFE 2.0
#define EPS 3.0e-7
#define FPMIN 1.0e-30
#define SHFT(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
static double maxarg1,maxarg2;
#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1) > (maxarg2) ? (maxarg1) : (maxarg2))
#define CGOLD 0.3819660

typedef struct {
	int counter;
	double xminimum[NDIM];
	double loglikelihood;
} kpsorter;

int structcomparison(const void *v1, const void *v2);

double betai(double a, double b, double x);
double betacf(double a, double b, double x);
double gammln(double xx);

/* Driver for routine powell */

void powell(double pp[], double **xi, int n, double ftol, int *iter, double *fret,
	    double (*func)(double []));
void linmin(double pp[], double xi[], int n, double *fret, double (*func)(double []));
void mnbrak(double *ax, double *bx, double *cx, double *fa, double *fb, double *fc,
	    double (*func)(double));
double brent(double ax, double bx, double cx, double (*f)(double), double tol,
	    double *xmin);
double f1dim(double x);

void amoeba(double **p, double y[], int ndim, double ftol,
	    double (*funk)(double []), int *nfunk);
double amotry(double **p, double y[], double psum[], int ndim,
	      double (*funk)(double []), int ihi, double fac);
double dfridr(double (*func)(double []), double x[], double h, double *err, int iiii);
double dfridr2nd(double (*func)(double []), double x[], double h, double *err, int iiii, int jjjj);
void xsvd(int kpnp, int kpnq, double *, double *, double *, double *);
double keithrules(double x[]);

double func(double x[])
{
        return keithrules(x);
}
int ncom;
double *pcom,*xicom,(*nrfunc)(double []);
static double *Y,*X;
FILE *fp;
FILE *jp;
FILE *kp;

int main(void)
{
	static double *pp;
	double xxx, vvv, aaa, bbb, ttt, tinvprob;
	int i,info,iter,j,iiii,jjjj,ntrials,errno;
	int nfunc,ndim=NDIM;
	double h,dx,dx2nd,err;
	double fret,**xi;
	double *x,*y,*ppjunk,**p,*firstderv,*secondderv;
	double *XXinv, *xxinvdiag;
	double *u, *lambda, *vt;
	double time1, time2, timedif;
	int *ipiv;
//	
	X = (double *) malloc (nrowX*ncolX*sizeof(double));
	Y = (double *) malloc (nrowX*sizeof(double));
	x = (double *) malloc ((NDIM+1)*sizeof(double));
	firstderv = (double *) malloc ((NDIM)*sizeof(double));
	secondderv = (double *) malloc ((NDIM)*(NDIM)*sizeof(double));
	y = (double *) malloc ((NDIM+1+1)*sizeof(double));
	XXinv = (double *) malloc (NDIM*NDIM*sizeof(double));
	ipiv = (int *) malloc (NDIM*sizeof(int));
	xxinvdiag = (double *) malloc (NDIM*sizeof(double));
	u      = (double *) malloc (NDIM*NDIM*sizeof(double));
	lambda = (double *) malloc (NDIM*NDIM*sizeof(double));
	vt     = (double *) malloc (NDIM*NDIM*sizeof(double));
/* Dynamic Allocation of a Matrix: */
/* First, allocate pointers to rows */
/* Second, allocate rows and set pointers to them -- note that p[0] is
 * a pointer to p[0][0:n] -- that is a row -- so the syntax below is
 * p[0][0:nrow*ncol] */
/* Third, form the matrix from step two -- the pointer to the kth row,
 * p[k] is set equal to the memory
 * location of the number of nrows-1 times the number of columns plus
 * the memory location of p[0] */
	p= (double **) malloc((NDIM+1+1)*sizeof(double*));
	p[0]=(double *) malloc((NDIM+1)*(NDIM+1+1)*sizeof(double));
	for (i=0;i<NDIM+1+1;i++)p[i]=p[0]+i*(NDIM+1);
//
	/* Dynamically allocate memory to big data structure
	 * ntrials is equal to the number of solutions from Amoeba --
	 * NDIM+1 plus the number of solutions from Powell -- in this
	 * case NDIM+1+10
	 */
	ntrials=NDIM+1+NDIM+1+10;
	kpsorter *recordset;
	recordset = (kpsorter *)malloc(ntrials * sizeof(kpsorter));
//
/* clock() is part of time.h -- returns the implementation's
 * best approximationto the processor time elapsed since the
 * program was invoke, divide by CLOCKS_PER_SEC to get the time
 * in seconds           */
	time1 = (double) clock();            /* get initial time */
	time1 = time1 / CLOCKS_PER_SEC;      /*    in seconds    */

	srand(122);
//
	jp = fopen("c:/docs_c_summer_course/data_optim.txt","w");
	kp = fopen("c:/docs_c_summer_course/data_optim_2.txt","w");
	if((fp = fopen("c:/docs_c_summer_course/oprobit_big_3.srt","r"))==NULL)
	{
		printf("\nUnable to open file BUSH2000.TXT: %s\n", strerror(errno));
		exit(EXIT_FAILURE);
	}
	else {

		fprintf(jp," Y and X = \n");
		for(i=0;i<nrowX;i++)
		{
			fscanf(fp,"%lf",&Y[i]);
			for(j=0;j<ncolX;j++)
			{
				fscanf(fp,"%lf",&X[i+j*nrowX]);
			}
			fprintf(jp,"%10d %12.6f", i,Y[i]);
			for(j=0;j<ncolX;j++)
			{
				fprintf(jp,"%12.6f",X[i+j*nrowX]);
			}
			fprintf(jp,"\n");
		}
	}
//
//  AMOEBA MINIMIZATION IS DONE FIRST
//
/* if i == (j+1) is true then the value of x[j] = 1.0, otherwise =0.0 */
	for (i=1;i<=NDIM+1;i++) {
		for (j=1;j<=NDIM;j++)
			x[j]=p[i][j]=(i == (j+1) ? 1.0 : 0.0);
		y[i]=func(x);
	}
	amoeba(p,y,ndim,FTOL,func,&nfunc);
	printf("\nNumber of function evaluations: %3d\n",nfunc);
	fprintf(jp,"\nNumber of function evaluations: %3d\n",nfunc);
	printf("Vertices of final 3-d simplex and\n");
	fprintf(jp,"Vertices of final 3-d simplex and\n");
	printf("function values at the vertices:\n\n");
	fprintf(jp,"function values at the vertices:\n\n");
	printf("%3s %10s %12s %12s %14s\n\n",
	       "i","x[i]","y[i]","z[i]","function");
	for (i=1;i<=NDIM+1;i++) {
		printf("%3d ",i);
		fprintf(jp,"%3d ",i);
		for (j=1;j<=NDIM;j++) {
			fprintf(jp,"%12.6f ",p[i][j]);
//
// store solution in x[]
//
                        x[j]=p[i][j];
			recordset[i-1].xminimum[j-1]=x[j];
		}
		printf("%12.6f\n",y[i]);
		fprintf(jp,"%12.6f\n",y[i]);
		recordset[i-1].loglikelihood=y[i];
		recordset[i-1].counter=i;
	}
//
//  POWELL MINIMIZATION IS DONE SECOND -- first, the NDIM+1 (ndim + 1)
//  solutions are checked and, second, 10 random starts are checked 
//
	for(iiii=1;iiii <= NDIM+1+10; iiii++)
	{
	pp = (double *) malloc ((NDIM+1)*sizeof(double));
	ppjunk = (double *) malloc ((NDIM+1)*sizeof(double));
	/*
	Get Uniform (-1, +1) Numbers for starts
	 */
	pp[0]=0.0;
	for(i=1;i < NDIM+1;i++)
	{
		if(iiii <= NDIM+1) pp[i] = p[iiii][i];
		if(iiii > NDIM+1) pp[i] = 2.0*( (double)rand() / ((double)(RAND_MAX)+1))-1.0;
		ppjunk[i] = pp[i];
	}
//
/* Dynamic Allocation of a Matrix: */
/* First, allocate pointers to rows */
/* Second, allocate rows and set pointers to them -- note that p[0] is
 * a pointer to p[0][0:n] -- that is a row -- so the syntax below is
 * p[0][0:nrow*ncol] */
/* Third, form the matrix from step two -- the pointer to the kth row,
 * p[k] is set equal to the memory
 * location of the number of nrows-1 times the number of columns plus
 * the memory location of p[0] */
	xi= (double **) malloc((NDIM+1)*sizeof(double*));
	xi[0]=(double *) malloc((NDIM+1)*(NDIM+1)*sizeof(double));
	for (i=0;i<NDIM+1;i++)xi[i]=xi[0]+i*(NDIM+1);
//
	for (i=1;i<=NDIM;i++)
		for (j=1;j<=NDIM;j++)
			xi[i][j]=(i == j ? 1.0 : 0.0);
/* if i == j is true then the value of x[i][j] = 1.0, otherwise =0.0 */
	powell(pp,xi,NDIM,FTOL,&iter,&fret,func);
	fprintf(jp,"Iterations: %3d\n\n",iter);
//	printf("Iterations: %3d\n\n",iter);
	fprintf(jp,"Minimum found at: \n");
//	printf("Minimum found at: \n");
	for (i=1;i<=NDIM;i++) {
		fprintf(jp,"%5d %12.6f %12.6f\n",i, pp[i], ppjunk[i]);
//		printf("%5d %12.6f %12.6f\n",i, pp[i], ppjunk[i]);
		recordset[iiii-1+NDIM+1].xminimum[i-1]=pp[i];
	}
	fprintf(jp,"\n\n%5d Minimum function value = %12.6f \n\n",iiii,fret);
	printf("%5d Minimum function value = %12.6f \n",iiii,fret);
	recordset[iiii-1+NDIM+1].loglikelihood=fret;
	recordset[iiii-1+NDIM+1].counter=iiii+NDIM+1;
	free(xi);
	free(pp);
	free(ppjunk);
}
	qsort(recordset, ntrials, sizeof(kpsorter), structcomparison);
	for(i=0;i<ntrials;i++)
	{
//
// Pass Best Solution to Numerical Derivatives
//
		if(i == 0){
			for(j=1;j<=NDIM;j++)
	        	{
			    x[j]=recordset[i].xminimum[j-1];
		        }
		}
		fprintf(kp,"%5d %5d  %14.6f", i, recordset[i].counter,recordset[i].loglikelihood);
		for(j=0;j<NDIM;j++)
		{
			fprintf(kp,"%12.6f",recordset[i].xminimum[j]);
		}
		fprintf(kp,"\n");
	}
	fprintf(kp,"\n\n");
	h = 0.01;
// 1st derivatives
	for(iiii=1;iiii<=NDIM;iiii++){
		h = 0.01;
		dx=dfridr(func,x,h,&err,iiii);
		fprintf(jp,"dfridr    = %5d %5d %12.6f %12.6f %12.6f\n",iiii,iiii,dx,h,err);
		firstderv[iiii-1]=dx;
// 2nd derivatives
		for(jjjj=1;jjjj<=NDIM;jjjj++){
			h = 0.01;
			dx2nd=dfridr2nd(func,x,h,&err,iiii,jjjj);
			fprintf(jp,"dfrid2nd  = %5d %5d %12.6f %12.6f %12.6f\n",iiii,jjjj,dx2nd,h,err);
			secondderv[iiii-1 + (jjjj-1)*NDIM]=dx2nd;
		}
	}
	fprintf(jp,"\n\nFirst Derivatives \n\n");
	for(iiii=0;iiii<NDIM;iiii++){
		fprintf(jp," %5d %12.6f %12.6f\n",iiii,firstderv[iiii],x[iiii+1]);
	}
	fprintf(jp,"\n\nSecond Derivatives \n\n");
	for(iiii=0;iiii<NDIM;iiii++){
		for(jjjj=0;jjjj<NDIM;jjjj++){
			fprintf(jp,"%14.6f",secondderv[iiii + (jjjj)*NDIM]);
		}
		fprintf(jp,"\n");
	}	
/* Call Singular Value Decomposition Routine to look at the rank 
 * the Hessian*/
/*  Clock the SVD Routine*/

	time2 = (double) clock();            /* get initial time */
	time2 = time2 / CLOCKS_PER_SEC;      /*    in seconds    */
	xsvd(NDIM,NDIM,secondderv,u,lambda,vt);
	printf("Singular Values\n");
	for(j=0;j<NDIM;j++)
	{
		printf("%5d %16.6f\n",j,lambda[j]);
	}
	timedif = ( ((double) clock()) / CLOCKS_PER_SEC) - time2;
	printf("SVD took %12.3f seconds\n", timedif);
	fprintf(jp,"SVD took %12.3f seconds\n", timedif);
	fprintf(kp,"SVD took %12.3f seconds\n", timedif);
/*
   Check to make certain the Hessian is non-singular
*/
	if(lambda[NDIM-1]<=TOL){
		fprintf(jp," Error: Hessian Matrix is Singular!");
		fprintf(kp," Error: Hessian Matrix is Singular!");
		printf(" Error: Hessian Matrix is Singular!");
	}
        if(lambda[NDIM-1]>TOL){
/* Initialize the Identity matrix */
	for(i=0;i<NDIM;i++)
	{
		for(j=0;j<NDIM;j++)
		{
			XXinv[i+j*NDIM]=0.0;
			if(i == j)XXinv[i+j*NDIM]=1.0;
		}
	}
	dgesv_(&ndim,&ndim,secondderv,&ndim,ipiv,XXinv,&ndim,&info);
	fprintf(kp,"\n\nInverse Hessian = \n");
	for(i=0;i<NDIM;i++)
	{
		for(j=0;j<NDIM;j++)
		{
			fprintf(kp,"%12.6f",XXinv[i+j*NDIM]);
/* Save Diagonal of Inverse Hessian */
			if(i == j)xxinvdiag[i] = XXinv[i+j*NDIM];
		}
		fprintf(kp,"\n");
	}
	fprintf(kp,"\n\nFirst Derivatives, Coefficients, and Standard Errors \n\n");
	vvv = (double)(nrowX-NDIM);
	aaa = vvv/2.0;
	bbb = 0.5;
	for(iiii=0;iiii<NDIM;iiii++){
		ttt = x[iiii+1]/sqrt(xxinvdiag[iiii]);
		xxx = vvv/(vvv+ttt*ttt);
		tinvprob = betai(aaa, bbb, xxx);
		fprintf(kp," %5d %12.6f %12.6f %12.6f %12.6f %12.6f\n",iiii,firstderv[iiii],x[iiii+1],sqrt(xxinvdiag[iiii]), ttt, tinvprob);
	}
	}
	free(Y);
	free(X);
	free(x);
	free(firstderv);
	free(secondderv);
	free(y);
	free(XXinv);
	free(ipiv);
	free(xxinvdiag);
	free(u);
	free(lambda);
	free(vt);
	free(p);
//
	timedif = ( ((double) clock()) / CLOCKS_PER_SEC) - time1;
	printf("The total elapsed time of the program is %12.3f seconds\n", timedif);
	fprintf(jp,"\nThe total elapsed time of the program is %12.3f seconds\n", timedif);
	fprintf(kp,"\nThe total elapsed time of the program is %12.3f seconds\n", timedif);

	fclose(kp);
	fclose(jp);
	fclose(fp);
	return 0;
}
void powell(double pp[], double **xi, int n, double ftol, int *iter, double *fret,
	    double (*func)(double []))
{
	int i,ibig,j;
	double del,fp,fptt,t,*pt,*ptt,*xit;

	pt = (double *) malloc ((n+1)*sizeof(double));
	ptt = (double *) malloc ((n+1)*sizeof(double));
	xit = (double *) malloc ((n+1)*sizeof(double));
	*fret=(*func)(pp);
	for (j=1;j<=n;j++) pt[j]=pp[j];
	for (*iter=1;;++(*iter)) {
		fp=(*fret);
		ibig=0;
		del=0.0;
		for (i=1;i<=n;i++) {
			for (j=1;j<=n;j++) xit[j]=xi[j][i];
			fptt=(*fret);
			linmin(pp,xit,n,fret,func);
			if (fptt-(*fret) > del) {
				del=fptt-(*fret);
				ibig=i;
			}
		}
		if (2.0*(fp-(*fret)) <= ftol*(fabs(fp)+fabs(*fret))+TINY) {
			free(xit);
			free(ptt);
			free(pt);
			return;
		}
		if (*iter == ITMAX)
		{
			printf("powell exceeding maximum iterations.");
			exit(EXIT_FAILURE);
		}
		for (j=1;j<=n;j++) {
			ptt[j]=2.0*pp[j]-pt[j];
			xit[j]=pp[j]-pt[j];
			pt[j]=pp[j];
		}
		fptt=(*func)(ptt);
		if (fptt < fp) {
			t=2.0*(fp-2.0*(*fret)+fptt)*(fp-(*fret)-del)*(fp-(*fret)-del)-del*(fp-fptt)*(fp-fptt);
			if (t < 0.0) {
				linmin(pp,xit,n,fret,func);
				for (j=1;j<=n;j++) {
					xi[j][ibig]=xi[j][n];
					xi[j][n]=xit[j];
				}
			}
		}
	}
}


void linmin(double pp[], double xi[], int n, double *fret, double (*func)(double []))
{
	int j;
	double xx,xmin,fx,fb,fa,bx,ax;

	ncom=n;
	pcom = (double *) malloc ((n+1)*sizeof(double));
	xicom = (double *) malloc ((n+1)*sizeof(double));
	nrfunc=func;
	for (j=1;j<=n;j++) {
		pcom[j]=pp[j];
		xicom[j]=xi[j];
	}
	ax=0.0;
	xx=1.0;
	mnbrak(&ax,&xx,&bx,&fa,&fx,&fb,f1dim);
	*fret=brent(ax,xx,bx,f1dim,TOL,&xmin);
	for (j=1;j<=n;j++) {
		xi[j] *= xmin;
		pp[j] += xi[j];
	}
	free(xicom);
	free(pcom);
}


void mnbrak(double *ax, double *bx, double *cx, double *fa, double *fb, double *fc,
	    double (*func)(double))
{
	double ulim,u,r,q,fu,dum;

	*fa=(*func)(*ax);
	*fb=(*func)(*bx);
	if (*fb > *fa) {
		SHFT(dum,*ax,*bx,dum)
				SHFT(dum,*fb,*fa,dum)
	}
	*cx=(*bx)+GOLD*(*bx-*ax);
	*fc=(*func)(*cx);
	while (*fb > *fc) {
		r=(*bx-*ax)*(*fb-*fc);
		q=(*bx-*cx)*(*fb-*fa);
		u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/
		(2.0*SIGN(FMAX(fabs(q-r),TINY),q-r));
		ulim=(*bx)+GLIMIT*(*cx-*bx);
		if ((*bx-u)*(u-*cx) > 0.0) {
			fu=(*func)(u);
			if (fu < *fc) {
				*ax=(*bx);
				*bx=u;
				*fa=(*fb);
				*fb=fu;
				return;
			} else if (fu > *fb) {
				*cx=u;
				*fc=fu;
				return;
			}
			u=(*cx)+GOLD*(*cx-*bx);
			fu=(*func)(u);
		} else if ((*cx-u)*(u-ulim) > 0.0) {
			fu=(*func)(u);
			if (fu < *fc) {
				SHFT(*bx,*cx,u,*cx+GOLD*(*cx-*bx))
						SHFT(*fb,*fc,fu,(*func)(u))
			}
		} else if ((u-ulim)*(ulim-*cx) >= 0.0) {
			u=ulim;
			fu=(*func)(u);
		} else {
			u=(*cx)+GOLD*(*cx-*bx);
			fu=(*func)(u);
		}
		SHFT(*ax,*bx,*cx,u)
				SHFT(*fa,*fb,*fc,fu)
	}
}

double brent(double ax, double bx, double cx, double (*f)(double), double tol,
	    double *xmin)
{
	int iter;
	double a,b,d,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
	double e=0.0;

	a=(ax < cx ? ax : cx);
	b=(ax > cx ? ax : cx);
	x=w=v=bx;
	fw=fv=fx=(*f)(x);
	for (iter=1;iter<=ITMAX;iter++) {
		xm=0.5*(a+b);
		tol2=2.0*(tol1=tol*fabs(x)+TINY);
		if (fabs(x-xm) <= (tol2-0.5*(b-a))) {
			*xmin=x;
			return fx;
		}
		if (fabs(e) > tol1) {
			r=(x-w)*(fx-fv);
			q=(x-v)*(fx-fw);
			p=(x-v)*q-(x-w)*r;
			q=2.0*(q-r);
			if (q > 0.0) p = -p;
			q=fabs(q);
			etemp=e;
			e=d;
			if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
				d=CGOLD*(e=(x >= xm ? a-x : b-x));
			else {
				d=p/q;
				u=x+d;
				if (u-a < tol2 || b-u < tol2)
				d=SIGN(tol1,xm-x);
			}
		} else {
			d=CGOLD*(e=(x >= xm ? a-x : b-x));
		}
		u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));
		fu=(*f)(u);
		if (fu <= fx) {
			if (u >= x) a=x; else b=x;
			SHFT(v,w,x,u)
					SHFT(fv,fw,fx,fu)
		} else {
			if (u < x) a=u; else b=u;
			if (fu <= fw || w == x) {
				v=w;
				w=u;
				fv=fw;
				fw=fu;
			} else if (fu <= fv || v == x || v == w) {
				v=u;
				fv=fu;
			}
		}
	}
	printf("Too many iterations in brent");
	exit(EXIT_FAILURE);
	*xmin=x;
	return fx;
}

double f1dim(double x)
{
	int j;
	double f,*xt;

	xt = (double *) malloc ((ncom+1)*sizeof(double));
	for (j=1;j<=ncom;j++) xt[j]=pcom[j]+x*xicom[j];
	f=(*nrfunc)(xt);
	free(xt);
	return f;
}

void amoeba(double **p, double y[], int ndim, double ftol,
	    double (*funk)(double []), int *nfunk)
{
	int i,ihi,ilo,inhi,j,mpts=ndim+1;
	double rtol,sum,swap,ysave,ytry,*psum;

	psum = (double *) malloc ((ndim+1)*sizeof(double));
	*nfunk=0;
	for (j=1;j<=ndim;j++) {
		for (sum=0.0,i=1;i<=mpts;i++) sum += p[i][j];
		psum[j]=sum;}
	for (;;) {
		ilo=1;
		ihi = y[1]>y[2] ? (inhi=2,1) : (inhi=1,2);
		for (i=1;i<=mpts;i++) {
			if (y[i] <= y[ilo]) ilo=i;
			if (y[i] > y[ihi]) {
				inhi=ihi;
				ihi=i;
			} else if (y[i] > y[inhi] && i != ihi) inhi=i;
		}
		rtol=2.0*fabs(y[ihi]-y[ilo])/(fabs(y[ihi])+fabs(y[ilo])+TINY);
		if (rtol < ftol) {
			swap= y[1];
			y[1]=y[ilo];
			y[ilo]=swap;
			for (i=1;i<=ndim;i++)
			{
				swap=p[1][i];
				p[1][i]=p[ilo][i];
				p[ilo][i]=swap;
			}
			break;
		}
		if (*nfunk >= NMAX) 
		{
			printf("NMAX exceeded");
			exit(EXIT_FAILURE);
		}
		*nfunk += 2;
		ytry=amotry(p,y,psum,ndim,funk,ihi,-1.0);
		if (ytry <= y[ilo])
			ytry=amotry(p,y,psum,ndim,funk,ihi,2.0);
		else if (ytry >= y[inhi]) {
			ysave=y[ihi];
			ytry=amotry(p,y,psum,ndim,funk,ihi,0.5);
			if (ytry >= ysave) {
				for (i=1;i<=mpts;i++) {
					if (i != ilo) {
						for (j=1;j<=ndim;j++)
							p[i][j]=psum[j]=0.5*(p[i][j]+p[ilo][j]);
						y[i]=(*funk)(psum);
					}
				}
				*nfunk += ndim;
				for (j=1;j<=ndim;j++) {
					for (sum=0.0,i=1;i<=mpts;i++) sum += p[i][j];
					psum[j]=sum;}
			}
		} else --(*nfunk);
	}
	free(psum);
}


double amotry(double **p, double y[], double psum[], int ndim,
	      double (*funk)(double []), int ihi, double fac)
{
	int j;
	double fac1,fac2,ytry,*ptry;

	ptry = (double *) malloc ((ndim+1)*sizeof(double));
	fac1=(1.0-fac)/ndim;
	fac2=fac1-fac;
	for (j=1;j<=ndim;j++) ptry[j]=psum[j]*fac1-p[ihi][j]*fac2;
	ytry=(*funk)(ptry);
	if (ytry < y[ihi]) {
		y[ihi]=ytry;
		for (j=1;j<=ndim;j++) {
			psum[j] += ptry[j]-p[ihi][j];
			p[ihi][j]=ptry[j];
		}
	}
	free(ptry);
	return ytry;
}
double dfridr(double (*func)(double []), double x[], double h, double *err, int iiii)
{
	int i,j, errno;
	double errt,fac,hh,**a,ans;
	double *xx, *xxx;
	xx = (double *) malloc ((NDIM+1)*sizeof(double));
	xxx = (double *) malloc ((NDIM+1)*sizeof(double));
	for(i=1;i<=NDIM;i++){
		xx[i]=x[i];
		xxx[i]=x[i];
	}

/* Dynamic Allocation of a Matrix: */
/* First, allocate pointers to rows */
/* Second, allocate rows and set pointers to them -- note that p[0] is
 * a pointer to p[0][0:n] -- that is a row -- so the syntax below is
 * p[0][0:nrow*ncol] */
/* Third, form the matrix from step two -- the pointer to the kth row,
 * p[k] is set equal to the memory
 * location of the number of nrows-1 times the number of columns plus
 * the memory location of p[0] */
	a= (double **) malloc((NTAB+1)*sizeof(double*));
	a[0]=(double *) malloc((NTAB+1)*(NTAB+1)*sizeof(double));
	for (i=0;i<NTAB+1;i++)a[i]=a[0]+i*(NTAB+1);
//
	hh=h;
	xx[iiii]=x[iiii]+hh;
	xxx[iiii]=x[iiii]-hh;
	a[1][1]=((*func)(xx)-(*func)(xxx))/(2.0*hh);
//	a[1][1]=((*func)(x+hh)-(*func)(x-hh))/(2.0*hh);
	*err=BIG;
	for (i=2;i<=NTAB;i++) {
		hh /= CON;
		xx[iiii]=x[iiii]+hh;
		xxx[iiii]=x[iiii]-hh;
		a[1][i]=((*func)(xx)-(*func)(xxx))/(2.0*hh);
//		a[1][i]=((*func)(x+hh)-(*func)(x-hh))/(2.0*hh);
		fac=CON*CON;
		for (j=2;j<=i;j++) {
			a[j][i]=(a[j-1][i]*fac-a[j-1][i-1])/(fac-1.0);
			fac=CON*CON*fac;
			errt=FMAX(fabs(a[j][i]-a[j-1][i]),fabs(a[j][i]-a[j-1][i-1]));
			if (errt <= *err) {
				*err=errt;
				ans=a[j][i];
			}
		}
		if (fabs(a[i][i]-a[i-1][i-1]) >= SAFE*(*err)) break;
	}
	free(a);
	free(xx);
	free(xxx);
	return ans;
}
double dfridr2nd(double (*func)(double []), double x[], double h, double *err, int iiii, int jjjj)
{
	int i,j, errno;
	double errt,fac,hh,**a,ans;
	double xpp,xpm,xmp,xmm;
	double *xx, *xxx, *xxxx, *xxxxx;
	xx = (double *) malloc ((NDIM+1)*sizeof(double));
	xxx = (double *) malloc ((NDIM+1)*sizeof(double));
	xxxx = (double *) malloc ((NDIM+1)*sizeof(double));
	xxxxx = (double *) malloc ((NDIM+1)*sizeof(double));
	for(i=1;i<=NDIM;i++){
		xx[i]=x[i];
		xxx[i]=x[i];
		xxxx[i]=x[i];
		xxxxx[i]=x[i];
	}

	a= (double **) malloc((NTAB+1)*sizeof(double*));
	a[0]=(double *) malloc((NTAB+1)*(NTAB+1)*sizeof(double));
	for (i=0;i<NTAB+1;i++)a[i]=a[0]+i*(NTAB+1);
//
	hh=h;
//xx is ++
//xxx is +-
//xxxx is -+
//xxxxx is --
//
//  dxdx
//
	if(jjjj == iiii){
		xx[iiii]=x[iiii]+2*hh;
		xxx[iiii]=x[iiii]-2*hh;
		xxxx[iiii]=x[iiii];
		xxxxx[iiii]=x[iiii];
		xpp=(*func)(xx);
		xpm=(*func)(xxx);
		xmp=(*func)(xxxx);
		xmm=(*func)(xxxxx);
		a[1][1]=(xpp+xpm-xmp-xmm)/(4.0*hh*hh);
	}
//
//dxdy
//
	if(jjjj != iiii){
		xx[iiii]=x[iiii]+hh;
		xx[jjjj]=x[jjjj]+hh;
		xxx[iiii]=x[iiii]+hh;
		xxx[jjjj]=x[jjjj]-hh;
		xxxx[iiii]=x[iiii]-hh;
		xxxx[jjjj]=x[jjjj]+hh;
		xxxxx[iiii]=x[iiii]-hh;
		xxxxx[jjjj]=x[jjjj]-hh;
		xpp=(*func)(xx);
		xpm=(*func)(xxx);
		xmp=(*func)(xxxx);
		xmm=(*func)(xxxxx);
		a[1][1]=(xpp-xpm-xmp+xmm)/(4.0*hh*hh);
	}
//	printf(" xpp xpm xmp xmm hh a[1][1] %lf %lf %lf %lf %lf %lf\n",xpp,xpm,xmp,xmm,hh,a[1][1]);
	*err=BIG;
	for (i=2;i<=NTAB;i++) {
		hh /= CON;
//
//  dxdx
//
		if(jjjj == iiii){
			xx[iiii]=x[iiii]+2*hh;
			xxx[iiii]=x[iiii]-2*hh;
			xxxx[iiii]=x[iiii];
			xxxxx[iiii]=x[iiii];
			xpp=(*func)(xx);
			xpm=(*func)(xxx);
			xmp=(*func)(xxxx);
			xmm=(*func)(xxxxx);
			a[1][i]=(xpp+xpm-xmp-xmm)/(4.0*hh*hh);
		}
//
//dxdy
//
		if(jjjj != iiii){
			xx[iiii]=x[iiii]+hh;
			xx[jjjj]=x[jjjj]+hh;
			xxx[iiii]=x[iiii]+hh;
			xxx[jjjj]=x[jjjj]-hh;
			xxxx[iiii]=x[iiii]-hh;
			xxxx[jjjj]=x[jjjj]+hh;
			xxxxx[iiii]=x[iiii]-hh;
			xxxxx[jjjj]=x[jjjj]-hh;
			xpp=(*func)(xx);
			xpm=(*func)(xxx);
			xmp=(*func)(xxxx);
			xmm=(*func)(xxxxx);
			a[1][i]=(xpp-xpm-xmp+xmm)/(4.0*hh*hh);
		}
//		printf(" xpp xpm xmp xmm hh a[1][i] %lf %lf %lf %lf %lf %lf\n",xpp,xpm,xmp,xmm,hh,a[1][i]);
		fac=CON*CON;
		for (j=2;j<=i;j++) {
			a[j][i]=(a[j-1][i]*fac-a[j-1][i-1])/(fac-1.0);
			fac=CON*CON*fac;
			errt=FMAX(fabs(a[j][i]-a[j-1][i]),fabs(a[j][i]-a[j-1][i-1]));
			if (errt <= *err) {
				*err=errt;
				ans=a[j][i];
			}
		}
		if (fabs(a[i][i]-a[i-1][i-1]) >= SAFE*(*err)) break;
	}
	free(a);
	free(xx);
	free(xxx);
	free(xxxx);
	free(xxxxx);
	return ans;
}
int structcomparison(const void *v1, const void *v2)
{
/* Again, "kpsorter" is a declaration of a type like "int" or "double"
 * Here we declare two pointers p1 and p2 of type "kpsorter" and set
 * them equal to the pointers v1 and v2.  Note the "(kpsorter *)" CASTS
 * v1 and v2 as pointers of type structure as defined by kpsorter above*/
	kpsorter *p1=(kpsorter *)v1;
	kpsorter *p2=(kpsorter *)v2;
/* The "->" is special to structures -- it is "aiming" at the data
 * element of the structure */
	if(p1->loglikelihood < p2->loglikelihood)
		return -1;
	else if (p1->loglikelihood == p2->loglikelihood)
		return 0;
	else
		return 1;
}
/*Singular Value Decomposition Subroutine

   */
	void xsvd(int kpnp, int kpnq, double *y, double *u, double *lambda, double *vt) {
/*   
*/

		double *a, *work;
		double sumulv, svd_error_sum, svd_error_sum_2;
		int i, j, jj;
		int  info = 12;
		int  lwork= kpnp*kpnp+kpnq*kpnq;
		int  lda,ldu,ldvt;

		a     = calloc( kpnp*kpnq, sizeof(double));
		work  = calloc( lwork, sizeof(double));


		lda  = kpnp;
		ldu  = kpnp;
		ldvt = kpnq;
		for (i=0;i<lwork;i++){
			work[i] = 0;
		}

		fprintf(jp,"lwork=%i\n",lwork);

		for (j=0;j<kpnq;j++) {
			for (i=0;i<kpnp;i++) {
				a[(j*kpnp)+i] = y[(j*kpnp)+i];
			}
		}

		dgesvd_("A","A", &kpnp, &kpnq, a, &lda, lambda,
			  u, &ldu, vt, &ldvt, work, &lwork, &info);

		fprintf(kp,"Info = %i\n",info);
		printf("Info = %i\n",info);
		fprintf(kp,"Singular Values\n");
		printf("Singular Values\n");
		for(jj=0;jj<kpnq;jj++)
		{
			fprintf(kp,"%5d %16.6f\n",jj,lambda[jj]);
			printf("%5d %16.6f\n",jj,lambda[jj]);
		}
/*
  Do simple check of SVD

*/
		svd_error_sum=0.0;
		svd_error_sum_2=0.0;
		for (i=0;i<kpnp;i++)
		{
			for (jj=0;jj<kpnq;jj++)
			{
				sumulv=0.0;
				for (j=0;j<kpnq;j++)
				{
					sumulv+=u[(j*kpnp)+i]*lambda[j]*vt[j+(jj*kpnq)];
				}
				svd_error_sum+=(y[i+(jj*kpnp)]-sumulv)*(y[i+(jj*kpnp)]-sumulv);
				svd_error_sum_2+=fabs(y[i+(jj*kpnp)]-sumulv);
			}
		}
		fprintf(kp,"SVD Error Check = %12.7g %12.7g\n",svd_error_sum,svd_error_sum_2);
		printf("SVD Error Check = %12.7g %12.7g\n",svd_error_sum,svd_error_sum_2);
		free(work);
		free(a);
	}
/* Incomplete Beta Distribution Function -- Use to obtain t and F
 * distribution Probabilities (p-values) */
	double betai(double a, double b, double x)
	{
		double betacf(double a, double b, double x);
		double gammln(double xx);
//	void nrerror(char error_text[]);
		double bt;

		if (x < 0.0 || x > 1.0)
		{
			printf("Bad x in routine betai");
			exit(EXIT_FAILURE);
		}
		if (x == 0.0 || x == 1.0) bt=0.0;
		else
			bt=exp(gammln(a+b)-gammln(a)-gammln(b)+a*log(x)+b*log(1.0-x));
		if (x < (a+1.0)/(a+b+2.0))
			return bt*betacf(a,b,x)/a;
		else
			return 1.0-bt*betacf(b,a,1.0-x)/b;
	}

	double betacf(double a, double b, double x)
	{
//	void nrerror(char error_text[]);
		int m,m2;
		double aa,c,d,del,h,qab,qam,qap;

		qab=a+b;
		qap=a+1.0;
		qam=a-1.0;
		c=1.0;
		d=1.0-qab*x/qap;
		if (fabs(d) < FPMIN) d=FPMIN;
		d=1.0/d;
		h=d;
		for (m=1;m<=ITMAX;m++) {
			m2=2*m;
			aa=m*(b-m)*x/((qam+m2)*(a+m2));
			d=1.0+aa*d;
			if (fabs(d) < FPMIN) d=FPMIN;
			c=1.0+aa/c;
			if (fabs(c) < FPMIN) c=FPMIN;
			d=1.0/d;
			h *= d*c;
			aa = -(a+m)*(qab+m)*x/((a+m2)*(qap+m2));
			d=1.0+aa*d;
			if (fabs(d) < FPMIN) d=FPMIN;
			c=1.0+aa/c;
			if (fabs(c) < FPMIN) c=FPMIN;
			d=1.0/d;
			del=d*c;
			h *= del;
			if (fabs(del-1.0) < EPS) break;
		}
		if (m > ITMAX)
		{
			printf("a or b too big, or ITMAX too small in betacf");
			exit(EXIT_FAILURE);
		}
		return h;
	}
	double gammln(double xx)
	{
		double x,y,tmp,ser;
		static double cof[6]={76.18009172947146,-86.50532032941677,
		24.01409824083091,-1.231739572450155,
		0.1208650973866179e-2,-0.5395239384953e-5};
		int j;

		y=x=xx;
		tmp=x+5.5;
		tmp -= (x+0.5)*log(tmp);
		ser=1.000000000190015;
		for (j=0;j<=5;j++) ser += cof[j]/++y;
		return -tmp+log(2.5066282746310005*ser/x);
	}


double keithrules(double x[])
{
	int i, j, k1, k0, l1, l0;
	double sum=0;
	double phi=0.0;
	double xphi=0.0;
	double sumsquared=0;
	k1=0;
	k0=0;
	for(i=0;i<nrowX;i++)
	{
		sum=0.0;
// Kludge for Intercept Term
//		sum=x[1];
		for(j=0;j<ncolX;j++)
		{
			sum=sum+X[i+j*nrowX]*x[j+1]; // Note Change Here
		}
		phi = (erf(fabs(sum)/sqrt(2.0)))/2.0 + 0.5;
		xphi = phi;
		if(sum < 0.0)phi=1.0-xphi;
		xphi = phi;
		if(xphi > 0.99999999)phi=0.9999999;
		if(xphi < 0.00000001)phi=0.0000001;
// Voted for Bush
		if(Y[i] == 1.0){
			l1=1;
			l0=0;
			k1=k1+1;
			sumsquared = sumsquared + log(phi);
		}
// Voted for Gore
		if(Y[i] != 1.0){
			l1=0;
			l0=1;
			k0=k0+1;
			sumsquared = sumsquared + log(1.0 - phi);
		}
	}
//	printf("%lf\n",-sumsquared);
	return -sumsquared;

}

